@@ -14,14 +14,14 @@ use fuser::{
 };
 #[cfg(feature = "abi-7-26")]
 use log::info;
-use log::LevelFilter;
 use log::{debug, warn};
+use log::{error, LevelFilter};
 use serde::{Deserialize, Serialize};
 use std::cmp::min;
 use std::collections::BTreeMap;
 use std::ffi::OsStr;
 use std::fs::{File, OpenOptions};
-use std::io::{BufRead, BufReader, Read, Seek, SeekFrom, Write};
+use std::io::{BufRead, BufReader, ErrorKind, Read, Seek, SeekFrom, Write};
 use std::os::raw::c_int;
 use std::os::unix::ffi::OsStrExt;
 use std::os::unix::fs::FileExt;
@@ -1309,7 +1309,7 @@ impl Filesystem for SimpleFS {
                 return;
             }
         };
-        if let Err(error_code) = self.insert_link(&req, new_parent, new_name, inode, attrs.kind) {
+        if let Err(error_code) = self.insert_link(req, new_parent, new_name, inode, attrs.kind) {
             reply.error(error_code);
         } else {
             attrs.hardlinks += 1;
@@ -1358,11 +1358,10 @@ impl Filesystem for SimpleFS {
                     self.write_inode(&attr);
                     let open_flags = if self.direct_io { FOPEN_DIRECT_IO } else { 0 };
                     reply.opened(self.allocate_next_file_handle(read, write), open_flags);
-                    return;
                 } else {
                     reply.error(libc::EACCES);
-                    return;
                 }
+                return;
             }
             Err(error_code) => reply.error(error_code),
         }
@@ -1498,11 +1497,10 @@ impl Filesystem for SimpleFS {
                     self.write_inode(&attr);
                     let open_flags = if self.direct_io { FOPEN_DIRECT_IO } else { 0 };
                     reply.opened(self.allocate_next_file_handle(read, write), open_flags);
-                    return;
                 } else {
                     reply.error(libc::EACCES);
-                    return;
                 }
+                return;
             }
             Err(error_code) => reply.error(error_code),
         }
@@ -1616,7 +1614,7 @@ impl Filesystem for SimpleFS {
                 if size == 0 {
                     reply.size(data.len() as u32);
                 } else if data.len() <= size as usize {
-                    reply.data(&data);
+                    reply.data(data);
                 } else {
                     reply.error(libc::ERANGE);
                 }
@@ -2032,7 +2030,7 @@ fn main() {
         .unwrap_or_default()
         .to_string();
 
-    fuser::mount2(
+    let result = fuser::mount2(
         SimpleFS::new(
             data_dir,
             matches.is_present("direct-io"),
@@ -2040,6 +2038,13 @@ fn main() {
         ),
         mountpoint,
         &options,
-    )
-    .unwrap();
+    );
+    if let Err(e) = result {
+        // Return a special error code for permission denied, which usually indicates that
+        // "user_allow_other" is missing from /etc/fuse.conf
+        if e.kind() == ErrorKind::PermissionDenied {
+            error!("{}", e.to_string());
+            std::process::exit(2);
+        }
+    }
 }
